# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from zope.component import getSiteManager
from zope.i18n import translate
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class ContactTypeVocabularyFactory:
    def __call__(self, context=None):
        values = [
            (
                "organization",
                _(
                    "Organization (administrative service, business, professional, sports club, association, etc.)"
                ),
            ),
            (
                "position",
                _(
                    "Position (mayor, alderman, advisor, director, head of department, etc.)"
                ),
            ),
            ("mission", _("Mission (passports, reception, parking, etc.)")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


ContactTypeVocabulary = ContactTypeVocabularyFactory()


class ContactTypeDeVocabularyFactory:
    def __call__(self, context=None):
        vocabulary = ContactTypeVocabularyFactory()(context)
        translated_terms = [
            SimpleTerm(
                value=term.value,
                token=term.token,
                title=translate(term.title, target_language="de"),
            )
            for term in vocabulary
        ]
        return SimpleVocabulary(translated_terms)


ContactTypeDeVocabulary = ContactTypeDeVocabularyFactory()


class PhoneTypeVocabularyFactory:
    def __call__(self, context=None):
        """vcard spec : cell, home, work, text, voice, fax, video, pager, textphone"""
        values = [
            ("fax", _("Fax")),
            ("cell", _("Mobile")),
            ("home", _("Personal phone")),
            ("work", _("Work phone")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


PhoneTypeVocabulary = PhoneTypeVocabularyFactory()


class MailTypeVocabularyFactory:
    def __call__(self, context=None):
        """vcard spec : home, work"""
        values = [
            ("home", _("Personal email")),
            ("work", _("Work email")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


MailTypeVocabulary = MailTypeVocabularyFactory()


class SiteTypeVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("facebook", _("Facebook")),
            ("instagram", _("Instagram")),
            ("linkedin", _("Linkedin")),
            ("pinterest", _("Pinterest")),
            ("twitter", _("Twitter")),
            ("website", _("Website")),
            ("youtube", _("Youtube")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


SiteTypeVocabulary = SiteTypeVocabularyFactory()


class FacilitiesVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("accessibility", _("Accessibility (PMR)")),
            ("defibrillator", _("Defibrillator")),
            ("drinking_water_point", _("Drinking water point")),
            ("useful_numbers", _("Useful numbers")),
            ("public_toilets", _("Public toilets")),
            ("free_wifi", _("Free WIFI")),
            ("bottle_bubbles", _("Bottle bubbles")),
            ("clothing_bubbles", _("Clothing bubbles")),
            ("container_parks", _("Container parks")),
            ("charging_stations", _("Charging stations")),
            ("organic_waste_containers", _("Organic waste containers")),
            ("residual_waste_containers", _("Residual waste containers")),
            ("pmc_waste_containers", _("PMC waste containers")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


FacilitiesVocabulary = FacilitiesVocabularyFactory()


class FacilitiesDeVocabularyFactory:
    def __call__(self, context=None):
        vocabulary = FacilitiesVocabularyFactory()(context)
        translated_terms = [
            SimpleTerm(
                value=term.value,
                token=term.token,
                title=translate(term.title, target_language="de"),
            )
            for term in vocabulary
        ]
        return SimpleVocabulary(translated_terms)


FacilitiesDeVocabulary = FacilitiesDeVocabularyFactory()


class EntitiesUIDsVocabularyFactory:
    def __call__(self, context=None):
        portal = api.portal.get()
        brains = api.content.find(
            context=portal,
            portal_type="imio.directory.Entity",
            sort_on="sortable_title",
        )
        terms = [SimpleTerm(value=b.UID, token=b.UID, title=b.Title) for b in brains]
        return SimpleVocabulary(terms)


EntitiesUIDsVocabulary = EntitiesUIDsVocabularyFactory()


class ContactCategoriesDeVocabularyFactory:
    def __call__(self, context=None):
        sm = getSiteManager()
        contact_categories_taxo = sm.queryUtility(
            ITaxonomy, name="collective.taxonomy.contact_category"
        )
        categories_voca = contact_categories_taxo.makeVocabulary("de").inv_data
        terms = [
            SimpleTerm(value=k, token=k, title=v) for k, v in categories_voca.items()
        ]
        return SimpleVocabulary(terms)


ContactCategoriesDeVocabulary = ContactCategoriesDeVocabularyFactory()



# -*- coding: utf-8 -*-

from imio.directory.core.rest.odwb_endpoint import OdwbEndpointGet
from imio.directory.core.rest.odwb_endpoint import OdwbEntitiesEndpointGet
from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from unittest.mock import MagicMock
from unittest.mock import patch

import requests
import unittest


class RestFunctionalTest(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            id="entity",
            title="Entity",
        )

    @patch("requests.post")
    def test_odwb_url_errors(self, mock_post):
        contact = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            id="contact",
            title="Contact",
        )
        # OdwbEndpointGet.test_in_staging_or_local = True
        mock_request = MagicMock()

        mock_post.side_effect = requests.exceptions.ConnectionError(
            "ODWB : Connection error occurred"
        )
        endpoint = OdwbEndpointGet(contact, mock_request)
        response = endpoint.reply()
        self.assertEqual(response, "ODWB : Connection error occurred")
        mock_post.side_effect = requests.exceptions.Timeout("ODWB : Request timed out")
        endpoint = OdwbEndpointGet(contact, mock_request)
        response = endpoint.reply()
        self.assertEqual(response, "ODWB : Request timed out")

        mock_post.side_effect = requests.exceptions.HTTPError(
            "ODWB : HTTP error occurred"
        )
        endpoint = OdwbEndpointGet(contact, mock_request)
        response = endpoint.reply()
        self.assertEqual(response, "ODWB : HTTP error occurred")

        mock_post.side_effect = Exception("ODWB : Unexpected error occurred")
        endpoint = OdwbEndpointGet(contact, mock_request)
        response = endpoint.reply()
        self.assertEqual(response, "ODWB : Unexpected error occurred")

    @patch("requests.post")
    def test_get_news_to_send_to_odwb(self, m):
        contact = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            id="contact",
            title="Contact",
        )

        entity2 = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            id="entity2",
            title="Entity 2",
        )

        contact2 = api.content.create(
            container=entity2,
            type="imio.directory.Contact",
            id="contact2",
            title="Contact 2",
        )

        api.content.transition(contact, "publish")
        endpoint = OdwbEndpointGet(self.portal, self.request)
        endpoint.reply()
        # 1 (published) contact is returned on self.portal
        self.assertEqual(len(endpoint.__datas__), 1)

        api.content.transition(contact2, "publish")
        endpoint = OdwbEndpointGet(self.portal, self.request)
        endpoint.reply()
        # 2 (published) contacts are returned on self.portal
        self.assertEqual(len(endpoint.__datas__), 2)

        for data in endpoint.__datas__:
            if data.get("geolocation", None) is not None:
                self.assertEqual(data.get("geolocation"), {"lat": 4.5, "lon": 45})
            if data.get("latitude", None) is not None:
                self.assertEqual(data.get("latitude"), 4.5)
            if data.get("longitude", None) is not None:
                self.assertEqual(data.get("longitude"), 45)

        # test endpoint on entity
        endpoint = OdwbEndpointGet(self.entity, self.request)
        endpoint.reply()
        # 1 (published) contact is returned on self.entity
        self.assertEqual(len(endpoint.__datas__), 1)

    @patch("requests.post")
    def test_get_entities_to_send_to_odwb(self, m):
        entity2 = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            id="entity2",
            title="Entity 2",
        )
        # On directory auth source, entities are not automatically published
        api.content.transition(self.entity, "publish")
        api.content.transition(entity2, "publish")
        # OdwbEntitiesEndpointGet.test_in_staging_or_local = True
        endpoint = OdwbEntitiesEndpointGet(self.portal, self.request)
        endpoint.reply()
        self.assertEqual(len(endpoint.__datas__), 2)

        api.content.transition(self.entity, "reject")
        endpoint = OdwbEntitiesEndpointGet(self.portal, self.request)
        endpoint.reply()
        self.assertEqual(len(endpoint.__datas__), 1)



# -*- coding: utf-8 -*-

from collective.geolocationbehavior.geolocation import IGeolocatable
from imio.directory.core.utils import get_entity_uid_for_contact
from imio.smartweb.common.faceted.utils import configure_faceted
from imio.smartweb.common.upgrades import upgrades
from imio.smartweb.common.utils import translate_vocabulary_term
from plone import api
from plone.formwidget.geolocation.geolocation import Geolocation
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest

import geopy
import logging
import os

logger = logging.getLogger("imio.directory.core")


def add_current_entity_on_contacts(context):
    brains = api.content.find(portal_type="imio.directory.Contact")
    for brain in brains:
        contact = brain.getObject()
        entity_uid = get_entity_uid_for_contact(contact)
        contact.selected_entities = [entity_uid]
        contact.reindexObject(idxs=["selected_entities"])
    logger.info(
        "Finished selected_entities filling for {} contacts".format(len(brains))
    )


def refresh_entities_faceted(context):
    request = getRequest()
    faceted_config_path = "{}/faceted/config/entity.xml".format(
        os.path.dirname(__file__)
    )
    brains = api.content.find(portal_type="imio.directory.Entity")
    for brain in brains:
        obj = brain.getObject()
        configure_faceted(obj, faceted_config_path)
        request.form = {
            "cid": "entity",
            "faceted.entity.default": obj.UID(),
        }
        handler = getMultiAdapter((obj, request), name="faceted_update_criterion")
        handler.edit(**request.form)
        logger.info("Faceted refreshed on {}".format(obj.Title()))


def geocode_all_contacts(  # NOQA
    context,
    geocode_default=True,
    geocode_empty=True,
    clear_no_address=True,
    verbose=True,
):
    default_latitude = api.portal.get_registry_record("geolocation.default_latitude")
    default_longitude = api.portal.get_registry_record("geolocation.default_longitude")

    # allow to change the API key in the registry (this has to be done ttw)
    api_key = api.portal.get_registry_record(
        "geolocation.imio.opencage_api_key", default="801c91558c1f4338a62a43561fc961ab"
    )

    no_location = 0
    default_location = 0
    no_address = 0

    # we use OpenCage with a temporary API key for this migration because
    # Nominatim is too limited for bulk use
    geolocator = geopy.geocoders.OpenCage(api_key=api_key, timeout=3)

    brains = api.content.find(portal_type="imio.directory.Contact")
    for brain in brains:
        obj = brain.getObject()
        coordinates = IGeolocatable(obj).geolocation
        street_parts = [
            obj.number and str(obj.number) or "",
            obj.street,
            obj.complement,
        ]
        street = " ".join(filter(None, street_parts))
        entity_parts = [
            obj.zipcode and str(obj.zipcode) or "",
            obj.city,
        ]
        entity = " ".join(filter(None, entity_parts))
        country = translate_vocabulary_term(
            "imio.smartweb.vocabulary.Countries", obj.country
        )
        address = " ".join(filter(None, [street, entity, country]))
        if not address:
            no_address += 1
            if clear_no_address:
                # if we have no address, clear geolocation
                if obj.geolocation is not None:
                    obj.geolocation = Geolocation("", "")
                    obj.reindexObject(idxs=["longitude", "latitude"])
                    logger.info(
                        f"Contact has no address : {obj.absolute_url()} --> cleared geolocation"
                    )
                continue

        location = None
        if coordinates is None or not all(
            [coordinates.latitude, coordinates.longitude]
        ):
            # contact has no geolocation, see if we can find one
            if geocode_empty:
                logger.info("Geocoding address (empty coordinates): {}".format(address))
                try:
                    location = geolocator.geocode(address)
                except Exception as e:
                    logger.error(
                        f"Error geocoding address {address} for contact {obj.absolute_url()}: {e}"
                    )
                    no_location += 1
                    continue
                no_location += 1
                if location is None:
                    # geolocator could not find a location
                    logger.info(
                        f"Contact had no location : {obj.absolute_url()} --> no geolocation found for {address}"
                    )
                    continue
                logger.info(
                    f"Contact had no location : {obj.absolute_url()} --> {location.latitude} / {location.longitude}"
                )
        elif (
            coordinates.latitude == default_latitude
            and coordinates.longitude == default_longitude  # NOQA
        ):
            # contact was automatically geolocated on default coordinates
            if geocode_default:
                logger.info(
                    "Geocoding address (default coordinates): {}".format(address)
                )
                try:
                    location = geolocator.geocode(address)
                except Exception as e:
                    logger.error(
                        f"Error geocoding address {address} for contact {obj.absolute_url()}: {e}"
                    )
                    no_location += 1
                    continue
                default_location += 1
                if location is None:
                    # geolocator could not find a location
                    logger.info(
                        f"Contact was located on default coordinates : {obj.absolute_url()} --> no geolocation found for {address}"
                    )
                    continue
                logger.info(
                    f"Contact was located on default coordinates : {obj.absolute_url()} --> {location.latitude} / {location.longitude}"
                )
        else:
            # contact already has a geolocation, do nothing
            if verbose:
                logger.info(f"Contact already has its location : {obj.absolute_url()}")
            continue
        if location:
            obj.geolocation = Geolocation(
                latitude=location.latitude, longitude=location.longitude
            )
            obj.reindexObject(idxs=["longitude", "latitude"])
    logger.info(
        f"Geocoding finished for {len(brains)} contacts: "
        f"{no_location} had no location, {default_location} were located on default coordinates, {no_address} had they had no address."
    )
    if geocode_empty:
        logger.info(f"Geocoded {no_location} contacts with empty coordinates.")
    if geocode_default:
        logger.info(
            f"Geocoded {default_location} contacts that were geolocated on default coordinates."
        )
    if clear_no_address:
        logger.info(
            f"Cleared geolocation for {no_address} contacts that had no address."
        )


def geocode_default_contacts(context):
    """
    Geocode contacts that are geolocated on default coordinates.
    """
    geocode_all_contacts(
        context, geocode_empty=False, clear_no_address=False, verbose=False
    )


def reindex_searchable_text(context):
    upgrades.reindex_searchable_text(context)


def add_translations_indexes(context):
    catalog = api.portal.get_tool("portal_catalog")

    new_indexes = ["translated_in_nl", "translated_in_de", "translated_in_en"]
    indexes = catalog.indexes()
    indexables = []
    for new_index in new_indexes:
        if new_index in indexes:
            continue
        catalog.addIndex(new_index, "BooleanIndex")
        indexables.append(new_index)
        logger.info(f"Added BooleanIndex for field {new_index}")
    if len(indexables) > 0:
        logger.info(f"Indexing new indexes {', '.join(indexables)}")
        catalog.manage_reindexIndex(ids=indexables)

    new_metadatas = ["title_fr", "title_nl", "title_de", "title_en"]
    metadatas = list(catalog.schema())
    must_reindex = False
    for new_metadata in new_metadatas:
        if new_metadata in metadatas:
            continue
        catalog.addColumn(new_metadata)
        must_reindex = True
        logger.info(f"Added {new_metadata} metadata")
    if must_reindex:
        logger.info("Reindexing catalog for new metadatas")
        catalog.clearFindAndRebuild()


def add_contact_category_index(context):
    catalog = api.portal.get_tool("portal_catalog")
    indexes = catalog.indexes()
    new_index = "taxonomy_contact_category_for_filtering"
    if new_index in indexes:
        return
    catalog.addIndex(new_index, "KeywordIndex")
    idx = catalog._catalog.getIndex(new_index)
    idx.indexed_attrs = ("taxonomy_contact_category",)
    logger.info(f"Added KeywordIndex {new_index}")
    logger.info(f"Indexing new index {new_index}")
    catalog.manage_reindexIndex(ids=[new_index])


def reindex_catalog(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.clearFindAndRebuild()


def remove_searchabletext_fr(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.manage_delIndex("SearchableText_fr")


def remove_title_description_fr(context):
    catalog = api.portal.get_tool("portal_catalog")
    catalog.delColumn("title_fr")
    catalog.delColumn("description_fr")


def fix_datagridfields_values(context):
    brains = api.content.find(portal_type="imio.directory.Contact")
    for brain in brains:
        contact = brain.getObject()

        def fix_type(contact, field_name):
            if getattr(contact, field_name) is None:
                return
            fixed = False
            lines = getattr(contact, field_name)
            for line in lines:
                if isinstance(line["type"], tuple) or isinstance(line["type"], list):
                    line["type"] = line["type"][0]
                    fixed = True
            if fixed:
                contact._p_changed = 1
                logger.info(f"Fixed {field_name} types for {contact.absolute_url()}")

        fix_type(contact, "phones")
        fix_type(contact, "mails")
        fix_type(contact, "urls")
        fix_type(contact, "private_phones")
        fix_type(contact, "private_mails")
        fix_type(contact, "private_urls")


def fix_missing_values_for_facilities_lists(context):
    catalog = api.portal.get_tool("portal_catalog")
    with api.env.adopt_user(username="admin"):
        brains = api.content.find(portal_type=["imio.directory.Contact"])
        for brain in brains:
            must_reindex = False
            if brain.facilities is None:
                must_reindex = True
            obj = brain.getObject()
            if hasattr(obj, "facilities") and obj.facilities is None:
                obj.facilities = []
                must_reindex = True
                logger.info(f"Fixed None list for Facilities on {obj.absolute_url()}")
            if must_reindex:
                catalog.catalog_object(obj, idxs=["facilities"])
                logger.info(f"Reindexed Facilities on {obj.absolute_url()}")


def reindex_solr(context):
    portal = api.portal.get()
    maintenance = portal.unrestrictedTraverse("@@solr-maintenance")
    maintenance.clear()
    maintenance.reindex()



# -*- coding: utf-8 -*-

from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from imio.directory.core.tests.utils import make_named_image
from imio.smartweb.common.interfaces import ICropping
from plone import api
from plone.app.imagecropping.interfaces import IImageCroppingUtils
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.namedfile.file import NamedBlobImage
from zope.component import getMultiAdapter

import unittest


class TestCropping(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity",
        )
        self.contact = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="Contact",
        )

    def test_cropping_adapter(self):
        adapter = ICropping(self.contact, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual(
            adapter.get_scales("image", self.request),
            ["portrait_affiche", "paysage_affiche", "carre_affiche"],
        )
        self.assertEqual(adapter.get_scales("logo", self.request), [])

    def test_uncroppable_fields(self):
        self.contact.logo = NamedBlobImage(**make_named_image())
        self.contact.image = NamedBlobImage(**make_named_image())
        adapter = IImageCroppingUtils(self.contact, alternate=None)
        self.assertIsNotNone(adapter)
        self.assertEqual(len(list(adapter._image_field_values())), 1)
        self.assertEqual(adapter.image_field_names(), ["image"])

    def test_cropping_view(self):
        self.contact.logo = NamedBlobImage(**make_named_image())
        self.contact.image = NamedBlobImage(**make_named_image())
        cropping_view = getMultiAdapter(
            (self.contact, self.request), name="croppingeditor"
        )
        self.assertEqual(len(list(cropping_view._scales("logo"))), 0)
        self.assertEqual(len(list(cropping_view._scales("image"))), 3)
        self.assertNotIn("Logo", cropping_view())
        self.assertIn("Lead Image", cropping_view())



# -*- coding: utf-8 -*-
from collective.schedulefield.behavior import ScheduledWithTitle
from datetime import datetime
from DateTime import DateTime
from imio.smartweb.common.rest.odwb import OdwbBaseEndpointGet
from imio.smartweb.common.utils import (
    activate_sending_data_to_odwb_for_staging as odwb_staging,
)
from imio.directory.core.contents import IEntity
from imio.directory.core.contents import IContact
from imio.directory.core.utils import get_entity_for_contact
from plone import api
from plone.formwidget.geolocation.geolocation import Geolocation
from Products.CMFPlone.interfaces.siteroot import IPloneSiteRoot

import json
import logging
import requests

logger = logging.getLogger("imio.events.core")


class OdwbEndpointGet(OdwbBaseEndpointGet):
    def __init__(self, context, request):
        imio_service = (
            "contacts-en-wallonie"
            if not odwb_staging()
            else "staging-contacts-en-wallonie"
        )
        pushkey = f"imio.directory.core.odwb_{imio_service}_pushkey"
        super(OdwbEndpointGet, self).__init__(context, request, imio_service, pushkey)

    def reply(self):
        if not super(OdwbEndpointGet, self).available():
            return
        url = f"{self.odwb_api_push_url}/{self.odwb_imio_service}/temps_reel/push/?pushkey={self.odwb_pushkey}"
        self.__datas__ = self.get_contacts()
        batched_lst = [
            self.__datas__[i : i + 1000] for i in range(0, len(self.__datas__), 1000)
        ]
        for elem in batched_lst:
            payload = json.dumps(elem)
            response_text = self.odwb_query(url, payload)
            logger.info(response_text)
        return response_text

    def get_contacts(self):
        lst_contacts = []
        if IPloneSiteRoot.providedBy(self.context) or IEntity.providedBy(self.context):
            brains = api.content.find(
                object_provides=IContact.__identifier__, review_state="published"
            )
            for brain in brains:
                if IEntity.providedBy(self.context):
                    if self.context.UID() not in brain.selected_entities:
                        continue
                contact_obj = brain.getObject()
                contact = Contact(contact_obj)
                lst_contacts.append(json.loads(contact.to_json()))
        elif IContact.providedBy(self.context):
            contact = Contact(self.context)
            lst_contacts.append(json.loads(contact.to_json()))
        return lst_contacts

    def remove(self):
        if not super(OdwbEndpointGet, self).available():
            return
        lst_contacts = []
        if IContact.providedBy(self.context):
            contact = Contact(self.context)
            lst_contacts.append(json.loads(contact.to_json()))
        url = f"{self.odwb_api_push_url}/{self.odwb_imio_service}/temps_reel/delete/?pushkey={self.odwb_pushkey}"
        payload = json.dumps(lst_contacts)
        headers = {"Content-Type": "application/json"}
        response = requests.request("POST", url, headers=headers, data=payload)
        return response.text


class Contact:

    def __init__(self, context):
        # 2126b6ef314a4b3d9712732d5f036afc,
        self.id = context.id
        self.type = context.type  # get an id...
        self.title = context.title
        self.subtitle = context.subtitle
        # [{'label': None, 'type': 'fax', 'number': '+323874325'}]
        self.contact_phones = context.phones
        # [{'label': None, 'type': 'perso', 'mail': 'kamoulox@perdu.com'}]
        self.contact_emails = context.mails
        self.vat_number = context.vat_number
        self.description = context.description
        self.taxonomy_contact_category = context.taxonomy_contact_category
        self.topics = context.topics
        self.facilities = context.facilities
        self.image = f"{context.absolute_url()}/@@images/image/preview"
        self.coordinates = context.geolocation
        self.address_street_name = context.street
        self.address_street_number = context.number
        self.address_postal_code = context.zipcode
        self.address_city = context.city
        self.address_country = context.country
        self.urls = context.urls
        self.schedule = context.schedule
        self.multi_schedule = context.multi_schedule
        entity = get_entity_for_contact(context)
        self.owner_id = entity.UID()
        self.owner_name = entity.Title()
        # DateTime(2024/02/14 13:59:7.829612 GMT+1),
        self.creation_datetime = context.creation_date
        # DateTime(2024/02/14 15:51:52.128648 GMT+1),
        self.modification_datetime = context.modification_date

        # self.street_number_complement = context.complement
        # self.title_de = context.title_de
        # self.title_en = context.title_en
        # self.title_nl = context.title_nl
        # self.subtitle_de = context.subtitle_de
        # self.subtitle_en = context.subtitle_en
        # self.subtitle_nl = context.subtitle_nl
        # self.description_de = context.description_de
        # self.description_en = context.description_en
        # self.description_nl = context.description_nl
        # self.latitude = context.geolocation.latitude if context.geolocation else None
        # self.longitude = context.geolocation.longitude if context.geolocation else None
        # send to odwb?
        # self.private_phones = context.private_phones
        # self.private_mails = context.private_mails
        # self.private_urls = context.private_urls
        # self.private_note = context.private_note
        # ['e578c03fb70e448492ece56495686eae']
        # self.selected_entities = context.selected_entities
        # self.iam = context.iam
        # self.subjects = context.subjects
        # self.language = context.language
        # solr
        # self.searchwords = context.searchwords
        # DateTime(2024/02/14 13:59:00 GMT+1),
        # self.effective_date = context.effective_date

    def to_json(self):
        return json.dumps(self.__dict__, cls=ContactEncoder)


class ContactEncoder(json.JSONEncoder):

    def default(self, attr):
        if isinstance(attr, Geolocation):
            return {
                "lon": attr.longitude,
                "lat": attr.latitude,
            }
        elif isinstance(attr, DateTime):
            iso_datetime = attr.ISO8601()
            return iso_datetime
        elif isinstance(attr, datetime):
            return attr.isoformat()
        elif isinstance(attr, ScheduledWithTitle):
            start_date = None
            end_date = None
            if len(attr.dates) > 0:
                start_date = attr.dates[0].start_date.isoformat()
                end_date = attr.dates[0].end_date.isoformat()
            schedule = {
                "title": attr.title,
                "dates": {"start_date": start_date, "end_date": end_date},
                "schedule": attr.schedule,
            }
            return schedule
        else:
            return super().default(attr)


class OdwbEntitiesEndpointGet(OdwbBaseEndpointGet):

    def __init__(self, context, request):
        imio_service = (
            "entites-des-contacts-en-wallonie"
            if not odwb_staging()
            else "staging-entites-des-contacts-en-wallonie"
        )
        pushkey = f"imio.directory.core.odwb_{imio_service}_pushkey"
        super(OdwbEntitiesEndpointGet, self).__init__(
            context, request, imio_service, pushkey
        )

    def reply(self):
        if not super(OdwbEntitiesEndpointGet, self).available():
            return
        lst_entities = []
        brains = api.content.find(
            object_provides=IEntity.__identifier__, review_state="published"
        )
        for brain in brains:
            entity = {}
            entity["UID"] = brain.UID
            entity["id"] = brain.id
            entity["entity_title"] = brain.Title
            lst_entities.append(entity)
        self.__datas__ = lst_entities
        url = f"{self.odwb_api_push_url}/{self.odwb_imio_service}/temps_reel/push/?pushkey={self.odwb_pushkey}"
        payload = json.dumps(self.__datas__)
        headers = {"Content-Type": "application/json"}
        response = requests.request("POST", url, headers=headers, data=payload)
        return response.text



# -*- coding: utf-8 -*-

from imio.directory.core.contents.entity.export import ordered_signifiant_columns
from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation.geolocation import Geolocation
from zope.component import queryMultiAdapter

import io
import pandas
import unittest
from unittest.mock import MagicMock


class TestExport(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_view_on_entity_without_contact(self):
        entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            id="entity",
        )
        view = queryMultiAdapter((entity, self.request), name="export")
        view()
        self.assertEqual(view.request.response.status, 204)

    def test_view_to_export_contacts(self):
        entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            id="entity",
        )
        contact1 = api.content.create(
            container=entity,
            type="imio.directory.Contact",
            id="contact1",
        )
        contact1.title = "Contact 1"
        contact1.type = "organization"
        contact1.iam = ["disabled_person", "newcomer"]
        contact1.private_phones = [
            {
                "label": "Private kamoulox phone",
                "type": "cell",
                "number": "+32496111111",
            }
        ]
        contact1.private_mails = [
            {
                "label": "Private kamoulox mail",
                "type": "home",
                "mail_address": "test@imio.be",
            }
        ]

        contact2 = api.content.create(
            container=entity,
            type="imio.directory.Contact",
            id="contact2",
        )
        contact2.title = "Contact 2"
        contact2.geolocation = Geolocation(50.2, 5.2)
        # "Commerces et entreprises"
        contact2.taxonomy_contact_category = ["cho96vl9ox"]
        contact3 = api.content.create(
            container=entity,
            type="imio.directory.Contact",
            id="contact3",
        )
        contact3.title = "Contact 3"
        contact3.geolocation = Geolocation(50.3, 5.3)
        contact3.schedule = schedule()
        automatically_populating_attributes = [
            "title_nl",
            "title_de",
            "title_en",
            "subtitle",
            "subtitle_nl",
            "subtitle_de",
            "subtitle_en",
            "description_nl",
            "description_de",
            "description_en",
            "street",
        ]
        contacts = [contact1, contact2, contact3]
        for column in ordered_signifiant_columns:
            if column in automatically_populating_attributes:
                for contact in contacts:
                    setattr(contact, column, f"{contact.id} : {column}")
        response_mock = MagicMock()
        response_mock.setHeader = MagicMock()
        response_mock.write = MagicMock()
        view = queryMultiAdapter((entity, self.request), name="export")
        view.request.response = response_mock
        view()
        response_mock.setHeader.assert_called_with("Content-Type", "text/csv")
        # Capture the written content
        written_content = response_mock.write.call_args[0][0]
        assert isinstance(written_content, bytes)
        csv_buffer = io.BytesIO(written_content)
        # Read the CSV data into a pandas DataFrame
        df = pandas.read_csv(csv_buffer, delimiter="|")

        # Check that the columns are present
        for column in ordered_signifiant_columns:
            self.assertIn(column, df.columns.to_list())

        for auto_attr in automatically_populating_attributes:
            for contact in contacts:
                self.assertIn(f"{contact.id} : {auto_attr}", df[auto_attr].to_list())

        type_contact1 = df.loc[df["title"] == "Contact 1", "type"].values[0]
        self.assertEqual(
            type_contact1,
            "['Organization (administrative service, business, professional, sports club, association, etc.)']",
        )
        iam_contact1 = df.loc[df["title"] == "Contact 1", "iam"].values[0]
        self.assertEqual(iam_contact1, "['Disabled person', 'Newcomer']")

        private_mails_contact1 = df.loc[
            df["title"] == "Contact 1", "private_mails"
        ].values[0]
        self.assertEqual(
            private_mails_contact1,
            "[{'label': 'Private kamoulox mail', 'type': 'home', 'mail_address': 'test@imio.be'}]",
        )

        private_phones_contact1 = df.loc[
            df["title"] == "Contact 1", "private_phones"
        ].values[0]
        self.assertEqual(
            private_phones_contact1,
            "[{'label': 'Private kamoulox phone', 'type': 'cell', 'number': '+32496111111'}]",
        )

        latitude_contact2 = df.loc[
            df["title"] == "Contact 2", "geolocation.latitude"
        ].values[0]
        self.assertEqual(latitude_contact2, 50.2)

        schedule_monday_comment_contact3 = df.loc[
            df["title"] == "Contact 3", "schedule.monday.comment"
        ].values[0]
        self.assertEqual(schedule_monday_comment_contact3, "Kamoulox")


def schedule():
    schedule = {
        "monday": {
            "comment": "Kamoulox",
            "morningstart": "8:00",
            "morningend": "12:00",
            "afternoonstart": "13:00",
            "afternoonend": "18:00",
        },
        "tuesday": {
            "comment": "",
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
        },
        "wednesday": {
            "comment": "",
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
        },
        "thursday": {
            "comment": "",
            "morningstart": "10:00",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "18:30",
        },
        "friday": {
            "comment": "",
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
        },
        "saturday": {
            "comment": "",
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
        },
        "sunday": {
            "comment": "Always closed sunday",
            "morningstart": "",
            "morningend": "",
            "afternoonstart": "",
            "afternoonend": "",
        },
    }
    return schedule



# -*- coding: utf-8 -*-

from imio.directory.core.rest.odwb_endpoint import OdwbEndpointGet
from imio.directory.core.utils import get_entity_for_contact
from imio.directory.core.utils import get_entity_uid_for_contact
from imio.smartweb.common.faceted.utils import configure_faceted
from imio.smartweb.common.interfaces import IAddress
from imio.smartweb.common.utils import geocode_object
from imio.smartweb.common.utils import remove_cropping
from plone import api
from plone.api.content import get_state
from Products.DCWorkflow.interfaces import IAfterTransitionEvent
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest
from zope.lifecycleevent import ObjectRemovedEvent
from zope.lifecycleevent.interfaces import IAttributes

import os
import transaction


def set_default_entity_uid(contact):
    contact.selected_entities = contact.selected_entities or []
    uid = get_entity_uid_for_contact(contact)
    if uid not in contact.selected_entities:
        contact.selected_entities = contact.selected_entities + [uid]
    contact.reindexObject(idxs=["selected_entities"])


def added_entity(obj, event):
    faceted_config_path = "{}/faceted/config/entity.xml".format(
        os.path.dirname(__file__)
    )
    configure_faceted(obj, faceted_config_path)
    request = getRequest()
    request.form = {
        "cid": "entity",
        "faceted.entity.default": obj.UID(),
    }
    handler = getMultiAdapter((obj, request), name="faceted_update_criterion")
    handler.edit(**request.form)


def added_contact(obj, event):
    container_entity = get_entity_for_contact(obj)
    set_uid_of_referrer_entities(obj, container_entity)
    if not obj.is_geolocated:
        # geocode only if the user has not already changed geolocation
        geocode_object(obj)


def modified_contact(obj, event):
    set_default_entity_uid(obj)

    if not hasattr(event, "descriptions") or not event.descriptions:
        return
    for d in event.descriptions:
        if not IAttributes.providedBy(d):
            # we do not have fields change description, but maybe a request
            continue
        if d.interface is IAddress and d.attributes:
            # an address field has been changed
            geocode_object(obj)
        elif "ILeadImageBehavior.image" in d.attributes:
            # we need to remove cropping information of previous image
            remove_cropping(
                obj, "image", ["portrait_affiche", "paysage_affiche", "carre_affiche"]
            )
    if get_state(obj) == "published":
        request = getRequest()
        endpoint = OdwbEndpointGet(obj, request)
        endpoint.reply()


def modified_entity(obj, event):
    mark_current_entity_in_contacts_from_other_entities(obj, event)


def moved_contact(obj, event):
    if event.oldParent == event.newParent and event.oldName != event.newName:
        # item was simply renamed
        return
    if type(event) is ObjectRemovedEvent:
        # We don't have anything to do if event is being removed
        return
    container_entity = get_entity_for_contact(obj)
    set_uid_of_referrer_entities(obj, container_entity)
    if event.oldParent is not None and get_state(obj) == "published":
        request = getRequest()
        endpoint = OdwbEndpointGet(obj, request)
        endpoint.reply()


def removed_entity(obj, event):
    try:
        brains = api.content.find(selected_entities=obj.UID())
    except api.exc.CannotGetPortalError:
        # This happen when we try to remove plone object
        return
    for brain in brains:
        contact = brain.getObject()
        contact.selected_entities = [
            uid for uid in contact.selected_entities if uid != obj.UID()
        ]
        contact.reindexObject(idxs=["selected_entities"])


def removed_contact(obj, event):
    request = getRequest()
    endpoint = OdwbEndpointGet(obj, request)
    endpoint.remove()


def published_contact_transition(obj, event):
    if not IAfterTransitionEvent.providedBy(event):
        return
    if event.new_state.id == "published":
        kwargs = dict(obj=obj)
        transaction.get().addAfterCommitHook(send_to_odwb, kws=kwargs)
    if event.new_state.id == "private" and event.old_state.id != event.new_state.id:
        request = getRequest()
        endpoint = OdwbEndpointGet(obj, request)
        endpoint.remove()


def send_to_odwb(trans, obj=None):
    request = getRequest()
    endpoint = OdwbEndpointGet(obj, request)
    endpoint.reply()


def mark_current_entity_in_contacts_from_other_entities(obj, event):
    changed = False
    entities_to_treat = []
    for d in event.descriptions:
        if not IAttributes.providedBy(d):
            # we do not have fields change description, but maybe a request
            continue
        if "populating_entities" in d.attributes:
            changed = True
            uids_in_current_entity = [
                rf.to_object.UID() for rf in obj.populating_entities
            ]
            old_uids = getattr(obj, "old_populating_entities", [])
            entities_to_treat = set(old_uids) ^ set(uids_in_current_entity)
            break
    if not changed:
        return
    for uid_entity in entities_to_treat:
        entity = api.content.get(UID=uid_entity)
        contact_brains = api.content.find(
            context=entity, portal_type="imio.directory.Contact"
        )
        for brain in contact_brains:
            contact = brain.getObject()
            if uid_entity in uids_in_current_entity:
                contact.selected_entities.append(obj.UID())
                contact._p_changed = 1
            else:
                contact.selected_entities = [
                    item for item in contact.selected_entities if item != obj.UID()
                ]
            contact.reindexObject(idxs=["selected_entities"])
    # Keep a copy of populating_entities
    obj.old_populating_entities = uids_in_current_entity


def set_uid_of_referrer_entities(obj, container_entity):
    obj.selected_entities = [container_entity.UID()]
    rels = api.relation.get(target=container_entity, relationship="populating_entities")
    if not rels:
        obj.reindexObject(idxs=["selected_entities"])
        return
    for rel in rels:
        obj.selected_entities.append(rel.from_object.UID())
        obj._p_changed = 1
    obj.reindexObject(idxs=["selected_entities"])



# -*- coding: utf-8 -*-

from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory

import unittest


class TestVocabularies(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_contact_types(self):
        factory = getUtility(
            IVocabularyFactory, "imio.directory.vocabulary.ContactTypes"
        )
        vocabulary = factory()
        self.assertEqual(len(vocabulary), 3)

    def test_phone_types(self):
        factory = getUtility(IVocabularyFactory, "imio.directory.vocabulary.PhoneTypes")
        vocabulary = factory()
        self.assertEqual(len(vocabulary), 4)

    def test_mail_types(self):
        factory = getUtility(IVocabularyFactory, "imio.directory.vocabulary.MailTypes")
        vocabulary = factory()
        self.assertEqual(len(vocabulary), 2)

    def test_site_types(self):
        factory = getUtility(IVocabularyFactory, "imio.directory.vocabulary.SiteTypes")
        vocabulary = factory()
        self.assertEqual(len(vocabulary), 7)

    def test_facilities(self):
        factory = getUtility(IVocabularyFactory, "imio.directory.vocabulary.Facilities")
        vocabulary = factory()
        self.assertEqual(len(vocabulary), 13)

    def test_entities_UIDs(self):
        entity1 = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity1",
        )
        entity2 = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity2",
        )
        contact1 = api.content.create(
            container=entity1,
            type="imio.directory.Contact",
            title="Contact1",
        )
        factory = getUtility(
            IVocabularyFactory, "imio.directory.vocabulary.EntitiesUIDs"
        )
        vocabulary = factory(contact1)
        self.assertEqual(len(vocabulary), 2)

        vocabulary = factory(self.portal)
        self.assertEqual(len(vocabulary), 2)
        ordered_entities = [a.title for a in vocabulary]
        self.assertEqual(ordered_entities, [entity1.title, entity2.title])
        entity1.title = "Z Change order!"
        entity1.reindexObject()
        vocabulary = factory(self.portal)
        ordered_entities = [a.title for a in vocabulary]
        self.assertEqual(ordered_entities, [entity2.title, entity1.title])



# -*- coding: utf-8 -*-

from imio.smartweb.common.config import VOCABULARIES_MAPPING

VOCABULARIES_MAPPING.update(
    {
        "contact_type": "imio.directory.vocabulary.ContactTypes",
        "facilities": "imio.directory.vocabulary.Facilities",
        "taxonomy_contact_category": "collective.taxonomy.contact_category",
    }
)



# -*- coding: utf-8 -*-

from collective.taxonomy.interfaces import ITaxonomy
from imio.smartweb.common.utils import translate_vocabulary_term
from plone import api
from Products.Five.browser import BrowserView
from zope.component import getSiteManager

import io
import json
import pandas

# "exceptional_closure",
# "multi_schedule",
ordered_signifiant_columns = [
    "title",
    "title_nl",
    "title_de",
    "title_en",
    "subtitle",
    "subtitle_nl",
    "subtitle_de",
    "subtitle_en",
    "description_nl",
    "description_de",
    "description_en",
    "street",
    "number",
    "zipcode",
    "city",
    "country",
    "country.title",
    "complement",
    "is_geolocated",
    "geolocation.latitude",
    "geolocation.longitude",
    "mails",
    "phones",
    "urls",
    "private_mails",
    "private_phones",
    "private_urls",
    "private_note",
    "vat_number",
    "schedule",
    "schedule.monday.morningstart",
    "schedule.monday.morningend",
    "schedule.monday.afternoonstart",
    "schedule.monday.afternoonend",
    "schedule.monday.comment",
    "schedule.tuesday.morningstart",
    "schedule.tuesday.morningend",
    "schedule.tuesday.afternoonstart",
    "schedule.tuesday.afternoonend",
    "schedule.tuesday.comment",
    "schedule.wednesday.morningstart",
    "schedule.wednesday.morningend",
    "schedule.wednesday.afternoonstart",
    "schedule.wednesday.afternoonend",
    "schedule.wednesday.comment",
    "schedule.thursday.morningstart",
    "schedule.thursday.morningend",
    "schedule.thursday.afternoonstart",
    "schedule.thursday.afternoonend",
    "schedule.thursday.comment",
    "schedule.friday.morningstart",
    "schedule.friday.morningend",
    "schedule.friday.afternoonstart",
    "schedule.friday.afternoonend",
    "schedule.friday.comment",
    "schedule.saturday.morningstart",
    "schedule.saturday.morningend",
    "schedule.saturday.afternoonstart",
    "schedule.saturday.afternoonend",
    "schedule.saturday.comment",
    "schedule.sunday.morningstart",
    "schedule.sunday.morningend",
    "schedule.sunday.afternoonstart",
    "schedule.sunday.afternoonend",
    "schedule.sunday.comment",
    "type",
    "facilities",
    "iam",
    "taxonomy_contact_category",
    "topics",
    "subject",
    "modified",
]


class ExportView(BrowserView):

    def __call__(self):
        self.lang = api.portal.get_current_language()
        datas = self.get_datas()
        items = datas.get("items", None)
        if not items:
            return self.request.response.setStatus(204)
        df = pandas.json_normalize(items)
        dataframe_columns = list(df)
        set1 = set(ordered_signifiant_columns)
        set2 = set(dataframe_columns)
        intersection = set1.intersection(set2)
        columns = sorted(
            intersection, key=lambda x: ordered_signifiant_columns.index(x)
        )
        df = df[columns]

        csv_buffer = io.BytesIO()
        df.to_csv(csv_buffer, index=False, encoding="utf-8", sep="|")

        self.request.response.setHeader(
            "Content-Disposition", 'attachment; filename="export.csv"'
        )
        self.request.response.setHeader("Content-Type", "text/csv")
        self.request.response.write(csv_buffer.getvalue())
        return self.request.response

    def get_datas(self):
        context_path = "/".join(self.context.getPhysicalPath())
        query = {
            "portal_type": "imio.directory.Contact",
            "path": {"query": context_path, "depth": -1},
            "review_state": ["published", "private"],
        }

        # Executing the query
        brains = api.portal.get_tool("portal_catalog")(query)
        datas = []
        for brain in brains:
            obj = brain.getObject()
            items = {}
            for attribute in ordered_signifiant_columns:
                try:
                    json.dumps(getattr(obj, attribute, None))
                except TypeError:
                    if attribute == "modified":
                        items[attribute] = getattr(obj, attribute)().strftime(
                            "%Y-%m-%d %H:%M:%S.%f"
                        )
                    else:
                        continue
                else:
                    if attribute not in items:
                        items[attribute] = getattr(obj, attribute, None)
                if isinstance(getattr(obj, attribute, None), dict):
                    dict_obj = getattr(obj, attribute, None)
                    for k, v in dict_obj.items():
                        for v, v_value in v.items():
                            items[f"schedule.{k}.{v}"] = v_value
                if (
                    attribute == "geolocation.latitude"
                    or attribute == "geolocation.longitude"
                ):
                    attributes = attribute.split(".")
                    first_attr = getattr(obj, attributes[0], None)
                    items[f"{attributes[0]}.{attributes[1]}"] = (
                        None
                        if first_attr is None
                        else getattr(first_attr, attributes[1], None)
                    )
                if attribute == "taxonomy_contact_category":
                    items["taxonomy_contact_category"] = self.get_taxonomy_label_by_id(
                        obj
                    )
                if attribute == "iam" and getattr(obj, attribute, None) is not None:
                    items["iam"] = self.get_vocabulary_label(
                        "imio.smartweb.vocabulary.IAm", items["iam"]
                    )
                if (
                    attribute == "facilities"
                    and getattr(obj, attribute, None) is not None
                ):
                    items["facilities"] = self.get_vocabulary_label(
                        "imio.directory.vocabulary.Facilities", items["facilities"]
                    )
                if attribute == "topics" and getattr(obj, attribute, None) is not None:
                    items["topics"] = self.get_vocabulary_label(
                        "imio.smartweb.vocabulary.Topics", items["topics"]
                    )
                if attribute == "type" and getattr(obj, attribute, None) is not None:
                    items["type"] = self.get_vocabulary_label(
                        "imio.directory.vocabulary.ContactTypes", items["type"]
                    )
            datas.append(items)
        return {"items": datas, "items_total": len(datas)}

    def get_taxonomy_label_by_id(self, obj):
        sm = getSiteManager()
        utility = sm.queryUtility(
            ITaxonomy, name="collective.taxonomy.contact_category"
        )
        categories = []
        for category in getattr(obj, "taxonomy_contact_category", []) or []:
            categories.append(
                utility.translate(category, context=obj, target_language=self.lang)
            )
        return categories

    def get_vocabulary_label(self, voc, ids):
        ids = [ids] if not isinstance(ids, list) else ids
        labels = []
        for id in ids:
            label = translate_vocabulary_term(voc, id, self.lang)
            labels.append(label)
        return labels



# -*- coding: utf-8 -*-

from collective.geolocationbehavior.geolocation import IGeolocatable
from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.i18n.utility import setLanguageBinding

import unittest


class TestIndexes(unittest.TestCase):
    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity",
        )

    def test_selected_entities_index(self):
        contact1 = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="Contact1",
        )
        catalog = api.portal.get_tool("portal_catalog")
        brain = api.content.find(UID=contact1.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertFalse(indexes.get("is_geolocated"))
        self.assertEqual(indexes.get("container_uid"), self.entity.UID())
        IGeolocatable(contact1).geolocation = Geolocation(
            latitude="4.5", longitude="45"
        )
        contact1.reindexObject(idxs=["is_geolocated"])
        brain = api.content.find(UID=contact1.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertTrue(indexes.get("is_geolocated"))

        contact2 = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="Contact2",
        )
        entity2 = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity2",
        )
        contact1.selected_entities = [self.entity.UID()]
        contact1.reindexObject()
        brains = api.content.find(selected_entities=self.entity.UID())
        lst = [brain.UID for brain in brains]
        self.assertEqual(lst, [contact1.UID(), contact2.UID()])

        contact2.selected_entities = [entity2.UID(), self.entity.UID()]
        contact2.reindexObject()
        brains = api.content.find(selected_entities=entity2.UID())
        lst = [brain.UID for brain in brains]
        self.assertEqual(lst, [contact2.UID()])

        brains = api.content.find(selected_entities=[entity2.UID(), self.entity.UID()])
        lst = [brain.UID for brain in brains]
        self.assertEqual(lst, [contact1.UID(), contact2.UID()])

        contact2.selected_entities = [entity2.UID()]
        contact2.reindexObject()
        brains = api.content.find(selected_entities=[entity2.UID(), self.entity.UID()])
        lst = [brain.UID for brain in brains]
        self.assertEqual(lst, [contact1.UID(), contact2.UID()])

        api.content.move(contact1, entity2)
        brain = api.content.find(UID=contact1.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("container_uid"), entity2.UID())

    def test_searchable_text_index(self):
        contact = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="Title",
        )

        # use French (taxonomy is only translated in French)
        self.request["set_language"] = "fr"
        setLanguageBinding(self.request)

        catalog = api.portal.get_tool("portal_catalog")
        brain = api.content.find(UID=contact.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("SearchableText"), ["title"])

        contact.subtitle = "Subtitle"
        contact.description = "Description"
        contact.taxonomy_contact_category = [
            "brog42mktt",  # Auberge de jeunesse
            "hlsm9bijb1",  # Alimentation
        ]
        contact.topics = ["agriculture"]

        contact.mails = [
            {"label": "kamouloxmail", "mail_address": "ka@moulox.be", "type": "home"},
            {"label": None, "mail_address": "ka@moulox2.be", "type": "work"},
            {"label": "kamouloxmail2", "mail_address": "ka@moulox2.be", "type": "work"},
        ]
        contact.phones = [
            {"label": "kamouloxphone", "phone_number": "+3223456789", "type": "home"},
            {"label": "kamouloxphone2", "phone_number": "+3212345678", "type": "work"},
            {"label": None, "phone_number": "+3291234567", "type": "work"},
        ]
        contact.reindexObject()

        brain = api.content.find(UID=contact.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("SearchableText"),
            [
                "title",
                "subtitle",
                "description",
                "agriculture",
                "commerces",
                "et",
                "entreprises",
                "hebergement",
                "auberge",
                "de",
                "jeunesse",
                "commerces",
                "et",
                "entreprises",
                "alimentation",
                "kamouloxmail",
                "kamouloxmail2",
                "kamouloxphone",
                "kamouloxphone2",
            ],
        )

        contact2 = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="Title",
        )
        brain = api.content.find(UID=contact2.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(indexes.get("SearchableText"), ["title"])

        contact2.description = "Description"
        contact2.topics = ["agriculture"]
        contact2.taxonomy_contact_category = ["cho96vl9ox"]
        contact2.reindexObject()

        brain = api.content.find(UID=contact2.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("SearchableText"),
            [
                "title",
                "description",
                "agriculture",
                "commerces",
                "et",
                "entreprises",
            ],
        )

        contact2.title_de = "Titel"
        contact2.description_de = "Descriptie"
        contact2.reindexObject()

        brain = api.content.find(UID=contact2.UID())[0]
        indexes = catalog.getIndexDataForRID(brain.getRID())
        self.assertEqual(
            indexes.get("SearchableText_de"),
            [
                "titel",
                "descriptie",
                "landwirtschaft",
                "geschafte",
                "und",
                "unternehmen",
            ],
        )
